==> Apache.NMS.AMQP-1.8.2-src.zip: src/NMS.AMQP/Message/NmsMapMessage.cs <==
﻿/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

using Apache.NMS.AMQP.Message.Facade;
using Apache.NMS.Util;

namespace Apache.NMS.AMQP.Message
{
    public class NmsMapMessage : NmsMessage, IMapMessage
    {
        private readonly INmsMapMessageFacade facade;
        private PrimitiveMapInterceptor map;

        public NmsMapMessage(INmsMapMessageFacade facade) : base(facade)
        {
            this.facade = facade;
        }

        public override bool IsReadOnly
        {
            get => base.IsReadOnly;
            set
            {
                if (map != null) 
                    map.ReadOnly = value;

                base.IsReadOnly = value;
            }
        }

        public IPrimitiveMap Body => map ?? (map = new PrimitiveMapInterceptor(this, facade.Map, IsReadOnly, true));

        public override string ToString()
        {
            return $"NmsMapMessage {{ {Facade} }}";
        }
        
        public override NmsMessage Copy()
        {
            NmsMapMessage copy = new NmsMapMessage(facade.Copy() as INmsMapMessageFacade);
            CopyInto(copy);
            return copy;
        }
    }
}

==> Apache.NMS.AMQP-2.4.0-src.zip: src/NMS.AMQP/Message/NmsMapMessage.cs <==
﻿/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

using System;
using Apache.NMS.AMQP.Message.Facade;
using Apache.NMS.Util;

namespace Apache.NMS.AMQP.Message
{
    public class NmsMapMessage : NmsMessage, IMapMessage
    {
        private readonly INmsMapMessageFacade facade;
        private PrimitiveMapInterceptor map;

        public NmsMapMessage(INmsMapMessageFacade facade) : base(facade)
        {
            this.facade = facade;
        }

        public override bool IsReadOnly
        {
            get => base.IsReadOnly;
            set
            {
                if (map != null) 
                    map.ReadOnly = value;

                base.IsReadOnly = value;
            }
        }

        public IPrimitiveMap Body => map ?? (map = new PrimitiveMapInterceptor(this, facade.Map, IsReadOnly, true));

        public override string ToString()
        {
            return $"NmsMapMessage {{ {Facade} }}";
        }
        
        public override NmsMessage Copy()
        {
            NmsMapMessage copy = new NmsMapMessage(facade.Copy() as INmsMapMessageFacade);
            CopyInto(copy);
            return copy;
        }
        
        public override bool IsBodyAssignableTo(Type type)
        {
            return !facade.HasBody() || type.IsAssignableFrom(typeof(IPrimitiveMap));
        }
        
        protected override T DoGetBody<T>() {
            if (!facade.HasBody()) {
                return default;
            }

            return (T) Body;
        }
    }
}
